# -*- coding: utf-8 -*-

##############################################################################
#   (c)2023 by Oberhesse  (contact: vuplus-support.org)
#   Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License (CC BY-NC-SA 3.0)
#   FREE FOR PERSONAL USE
#   IF YOU WANT TO PUBLISH OR POST THIS PLUGIN, CONTACT THE AUTHOR
#   PLEASE DO NOT PUBLISH ON GITHUB
##############################################################################

from Components.Language import language
from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_LANGUAGE
from Components.config import config
import gettext
import os
from enigma import getDesktop


def localeInit():
    lang = language.getLanguage()
    try:
        os.environ["LANGUAGE"] = lang[:2]
    except:
        pass
    gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
    gettext.textdomain("enigma2")
    gettext.bindtextdomain("PasswordMasker", "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/PasswordMasker/locale/"))


def _(txt, DE=''):
    t = gettext.dgettext("PasswordMasker", txt)
    if t == txt:
        t = gettext.gettext(txt)
    if (t == txt) and DE:
        try:
            if str(config.osd.language.value).lower().startswith('de'):
                t = DE
        except:
            pass
    return t


try:
    localeInit()
    language.addCallback(localeInit)
except:
    pass


isPython3 = False
isDreamOS = False


try:
    import sys
    if sys.version_info[0] >= 3:
        isPython3 = True
except:
    pass
try:
    from enigma import eMediaDatabase
    isDreamOS = True
except:
    pass


try:
    scale = getDesktop(0).size().width() / 1280.0
except:
    scale = 1.0


def scaled(i):
    return int(round(i * scale))


def iconFile():
    return 'PasswordMasker.png' if scale <= 1 else 'PasswordMaskerFHD.png'


def scaledSkin(skin):

    def scaleVal(s):
        try:
            return scaled(int(s))
        except:
            return 0

    def sPos(s, text):
        i = text.find(s)
        return [i, 999999][(i < 0) or (s == '')]

    def firstElemInText(elem1, elem2, elem3, text):
        i = min(sPos(elem1, text), sPos(elem2, text), sPos(elem3, text))
        return [i, -1][i == 999999]
    res = ''
    while len(skin):
        i = firstElemInText('"_', ',_', ';_', skin)
        if (i < 0):
            return res + skin
        res += skin[:(i + 1)]
        skin = skin[(i + 2):]
        i = firstElemInText('"', ',', '', skin)
        if i < 0:
            return res + skin
        res += str(scaleVal(skin[:i]))
        skin = skin[i:]
    return res + skin
