# -*- coding: utf-8 -*-

##############################################################################
#   (c)2023 by Oberhesse  (contact: vuplus-support.org)
#   Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License (CC BY-NC-SA 3.0)
#   FREE FOR PERSONAL USE
#   IF YOU WANT TO PUBLISH OR POST THIS PLUGIN, CONTACT THE AUTHOR
#   PLEASE DO NOT PUBLISH ON GITHUB
##############################################################################


from . import _, scaledSkin, iconFile, isPython3, isDreamOS  # , scaled
from Plugins.Plugin import PluginDescriptor
from Components.ActionMap import ActionMap
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigPassword, ConfigText, ConfigYesNo
from Components.ConfigList import ConfigListScreen
from Screens.Screen import Screen
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Components.Label import Label
from datetime import datetime, timedelta
# import os


def debug(s, flag="a"):
    pass
    # f=open( "/usr/lib/enigma2/python/Plugins/Extensions/PasswordMasker/debug.txt", flag);  f.write( str(s)+'\n'); f.close()


pname = "Password Masker"
pdesc = "Makes Passwords unreadable on Screen"
pversion = "0.5-Beta"
pdate = "20240504"
pauthor = "Oberhesse"
license = "©2023 by Oberhesse (contact: vuplus-support.org)"

redirVK = False

config.plugins.PasswordMasker = ConfigSubsection()
config.plugins.PasswordMasker.MaskPW = ConfigYesNo(default=True)
config.plugins.PasswordMasker.MaskVKB = ConfigYesNo(default=True)
config.plugins.PasswordMasker.NoMaskWhenNew = ConfigYesNo(default=True)
config.plugins.PasswordMasker.MenuMode = ConfigSelection(default='0',  choices=[(
    "0", _("Extensions")), ("1", _("System Menu")), ("9", _("Extensions & System Menu"))])
config.plugins.PasswordMasker.PasswordTest = ConfigPassword(default='Password', fixed_size=False)


class MaskedLabel (Label):
    def __init__(self, text='', cursor='', *args, **kwargs):
        self.cursor = cursor
        self.unMasked = None
        text = text.replace('XcursorX', cursor)
        self.isEmptyPassword = (text.replace(self.cursor, '') == '')
        self.startTime = datetime.now()
        Label.__init__(self, '')
        self.setText(text)

    def setText(self, text):
        try:
            if self.isEmptyPassword:
                masked = text
            else:
                masked = ''
                if self.cursor:  # VTi Blinking Cursor
                    blank = ' ' * len(self.cursor)
                    temp1, temp2 = chr(1), chr(2)
                    text = text.replace('XcursorX', temp1).replace(self.cursor, temp1).replace(blank, temp2)
                    for c in list(text):
                        if c == temp1:
                            masked += self.cursor
                        elif c == temp2:
                            masked += blank
                        else:
                            masked += '*'
                else:
                    masked = "*" * len(text)
                if (masked == '') and config.plugins.PasswordMasker.NoMaskWhenNew.value:
                    if datetime.now() > self.startTime + timedelta(seconds=1):
                        self.isEmptyPassword = True
            self.message = str(masked)
            if self.instance:
                self.instance.setText(self.message or "")
            self.unMasked = text
        except Exception as e:
            debug(e)

    def getText(self):
        return self.unMasked if self.unMasked is not None else self.message


def newLayoutFinished(self, *args, **kwargs):
    self.pworig_layoutFinished(*args, **kwargs)
    global redirVK
    if redirVK:
        if (not config.plugins.PasswordMasker.NoMaskWhenNew.value) or (self["text"].getText() != ''):
            try:
                self["text"].type = 1
                self["text"].update()
            except:
                pass


def newVKInit(self, *args, **kwargs):
    self.pworig_init(*args, **kwargs)
    global redirVK
    if redirVK:
        try:
            if not hasattr(self, 'gui_cursor'):
                self.gui_cursor = ''
            if not isPython3:
                self["text"] = MaskedLabel(self.text, self.gui_cursor)
        except Exception as e:
            debug(e)


def newOnSelect(self, *args, **kwargs):
    try:
        global redirVK
        redirVK = config.plugins.PasswordMasker.MaskVKB.value
        if redirVK:
            vkInit()
        try:
            isEmpty = str(self.text) == ''
        except:
            isEmpty = False
        doShow = not config.plugins.PasswordMasker.MaskPW.value
        if isEmpty and not doShow:
            doShow = config.plugins.PasswordMasker.NoMaskWhenNew.value
        if doShow:
            self.pworig_onSelect(*args, **kwargs)
        elif isDreamOS:
            self.pworig_onSelect(*args, **kwargs)
            self.selected = False
        else:
            ConfigText.onSelect(self, args[0])
    except Exception as e:
        debug(e)


def newOnDeselect(self, *args, **kwargs):
    try:
        global redirVK
        redirVK = False
        self.pworig_onDeselect(*args, **kwargs)
    except:
        pass


def newGetMulti(self, *args, **kwargs):  # VTi Fix
    res = self.pworig_getMulti(*args, **kwargs)
    try:
        if isinstance(res, tuple):
            lst = list(res)
            if len(lst) >= 3:
                t = lst[1]
                if len(t) == len(self.text) + 1:
                    t = t[: len(self.text)]
                    lst[1] = t
                    res = tuple(lst)
    except:
        pass
    return res


class PasswordMaskerSettings (Screen, ConfigListScreen):

    skin = """
            <screen name="PasswordMasker Settings" position="center,center" size="_820,_272" title="$title$" >
                <widget name="config" font="Regular;_23"  position="_20,_23" size="_780,_180" itemHeight="_36" />
                <eLabel text="$version$" font="Regular;_19" position="_20,_230" size="_780,_20" halign="right" valign="center"  transparent="1" />
                <eLabel position="_27,_230" size="_17,_17" backgroundColor="#991111" transparent="0" />
                <eLabel text="$close$" font="Regular;_19" position="_53,_230" size="_200,_20" halign="left" valign="center"  transparent="1"  />
                <eLabel position="_197,_230" size="_17,_17" backgroundColor="#008800" transparent="0" />
                <eLabel text="$save$" font="Regular;_19" position="_223,_230" size="_200,_20" halign="left" valign="center"  transparent="1"  />
            </screen>
            """

    def __init__(self, session, args=0):
        self.session = session
        Screen.__init__(self, session)
        self.list = []
        self["actions"] = ActionMap(["OkCancelActions", "ColorActions"],
                                    {"cancel": self.keyCancel,
                                     "red": self.keyCancel,
                                     "green": self.doSave}, -1)
        if isDreamOS:
            self.skin = self.skin.replace('"config" font="Regular;_23" ', '"config" ')
        self.skin = scaledSkin(self.skin)
        self.list.append(getConfigListEntry(_('Mask selected Passwords in Configlist'), config.plugins.PasswordMasker.MaskPW))
        s = 'Hide' if isPython3 else 'Mask'
        self.list.append(getConfigListEntry(_(s + ' Passwords in Virtualkeyboard'), config.plugins.PasswordMasker.MaskVKB))
        self.list.append(getConfigListEntry(_('No Mask if Password is new'), config.plugins.PasswordMasker.NoMaskWhenNew))
        self.list.append(getConfigListEntry(_('Plugin Integration'), config.plugins.PasswordMasker.MenuMode))
        self.list.append(getConfigListEntry(_('Test Password'), config.plugins.PasswordMasker.PasswordTest))
        self.skin = self.skin.replace('$title$', pname + ' (' + _('Settings') + ')')
        self.skin = self.skin.replace('$info$',  _(''))
        self.skin = self.skin.replace('$version$', _('Version') + ' ' + pversion + '  -  © 2023 OH')
        self.skin = self.skin.replace('$save$', _('Save'))
        self.skin = self.skin.replace('$close$', _('Close'))

        ConfigListScreen.__init__(self, self.list, session=self.session)
        self.onLayoutFinish.append(self.startMenuMode)

    def doSave(self, x=None):
        self.saveAll()

    def startMenuMode(self):
        try:
            inst = self["config"].instance
            if hasattr(inst, 'setWrapAround'):
                inst.setWrapAround(True)
            if hasattr(inst, 'alignCenter') and hasattr(inst, 'setVAlign'):
                inst.setVAlign(inst.alignCenter)
            if hasattr(inst, 'alignCenter') and hasattr(inst, 'setVAlignSecond'):
                inst.setVAlignSecond(inst.alignCenter)
        except:
            pass


def vkInit():
    if not hasattr(VirtualKeyBoard, 'pworig_init'):
        setattr(VirtualKeyBoard, 'pworig_init', getattr(VirtualKeyBoard, '__init__'))
        setattr(VirtualKeyBoard, '__init__', newVKInit)
        if isPython3:
            setattr(VirtualKeyBoard, 'pworig_layoutFinished', getattr(VirtualKeyBoard, 'layoutFinished'))
            setattr(VirtualKeyBoard, 'layoutFinished', newLayoutFinished)


def init():
    try:
        if not hasattr(ConfigPassword, 'pworig_onSelect'):
            setattr(ConfigPassword, 'pworig_onSelect', getattr(ConfigPassword, 'onSelect'))
            setattr(ConfigPassword, 'onSelect', newOnSelect)
        if not hasattr(ConfigPassword, 'pworig_onDeselect'):
            setattr(ConfigPassword, 'pworig_onDeselect', getattr(ConfigPassword, 'onDeselect'))
            setattr(ConfigPassword, 'onDeselect', newOnDeselect)
        if (not isPython3) and (not isDreamOS):
            if not hasattr(ConfigPassword, 'pworig_getMulti'):
                setattr(ConfigPassword, 'pworig_getMulti', getattr(ConfigPassword, 'getMulti'))
                setattr(ConfigPassword, 'getMulti', newGetMulti)
    except Exception as e:
        debug(e)


def sessionstart(reason, **kwargs):
    if reason == 0:
        init()


def main(session, **kwargs):
    try:
        session.open(PasswordMaskerSettings)
    except:
        pass


def menuStart(menuid, **kwargs):
    if menuid == "system":
        return [(_(pname), main, _(pname), None)]
    else:
        return []


def Plugins(**kwargs):
    menuMode = config.plugins.PasswordMasker.MenuMode.value
    res = [PluginDescriptor(name=_(pname), where=PluginDescriptor.WHERE_SESSIONSTART, fnc=sessionstart)]
    if menuMode in ['0', '9']:
        res.append(PluginDescriptor(
                   name=_(pname), description=_(pdesc), where=PluginDescriptor.WHERE_PLUGINMENU,
                   icon=iconFile(), needsRestart=True, fnc=main))
    if menuMode in ['1', '9']:
        res.append(PluginDescriptor(
                   name=_(pname), description=_(pdesc), where=PluginDescriptor.WHERE_MENU,
                   icon=iconFile(), needsRestart=True, fnc=menuStart))
    return res
